package java.beans.beancontext;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.beans.VetoableChangeListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeSupport;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

public class BeanContextChildSupport implements BeanContextChild, BeanContextServicesListener, Serializable
{
    static final long serialVersionUID = 6328947014421475877L;
    public BeanContextChild beanContextChildPeer;
    protected PropertyChangeSupport pcSupport;
    protected VetoableChangeSupport vcSupport;
    protected transient BeanContext beanContext;
    protected transient boolean rejectedSetBCOnce;
    
    public BeanContextChildSupport() {
        this.beanContextChildPeer = this;
        this.pcSupport = new PropertyChangeSupport(this.beanContextChildPeer);
        this.vcSupport = new VetoableChangeSupport(this.beanContextChildPeer);
    }
    
    public BeanContextChildSupport(BeanContextChild beanContextChild) {
        this.beanContextChildPeer = ((beanContextChild != null) ? beanContextChild : this);
        this.pcSupport = new PropertyChangeSupport(this.beanContextChildPeer);
        this.vcSupport = new VetoableChangeSupport(this.beanContextChildPeer);
    }
    
    @Override
    public synchronized void setBeanContext(BeanContext beanContext) throws PropertyVetoException {
        if (beanContext == this.beanContext) {
            return;
        }
        BeanContext beanContext2 = this.beanContext;
        if (!this.rejectedSetBCOnce) {
            if ((this.rejectedSetBCOnce = !this.validatePendingSetBeanContext(beanContext))) {
                throw new PropertyVetoException("setBeanContext() change rejected:", new PropertyChangeEvent(this.beanContextChildPeer, "beanContext", beanContext2, beanContext));
            }
            try {
                this.fireVetoableChange("beanContext", beanContext2, beanContext);
            }
            catch (PropertyVetoException ex) {
                this.rejectedSetBCOnce = true;
                throw ex;
            }
        }
        if (this.beanContext != null) {
            this.releaseBeanContextResources();
        }
        this.beanContext = beanContext;
        this.rejectedSetBCOnce = false;
        this.firePropertyChange("beanContext", beanContext2, beanContext);
        if (this.beanContext != null) {
            this.initializeBeanContextResources();
        }
        return;
    }
    
    @Override
    public synchronized BeanContext getBeanContext() {
        return this.beanContext;
    }
    
    @Override
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.pcSupport.addPropertyChangeListener(propertyName, listener);
    }
    
    @Override
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.pcSupport.removePropertyChangeListener(propertyName, listener);
    }
    
    @Override
    public void addVetoableChangeListener(String propertyName, VetoableChangeListener listener) {
        this.vcSupport.addVetoableChangeListener(propertyName, listener);
    }
    
    @Override
    public void removeVetoableChangeListener(String propertyName, VetoableChangeListener listener) {
        this.vcSupport.removeVetoableChangeListener(propertyName, listener);
    }
    
    @Override
    public void serviceRevoked(BeanContextServiceRevokedEvent beanContextServiceRevokedEvent) {
    }
    
    @Override
    public void serviceAvailable(BeanContextServiceAvailableEvent beanContextServiceAvailableEvent) {
    }
    
    public BeanContextChild getBeanContextChildPeer() {
        return this.beanContextChildPeer;
    }
    
    public boolean isDelegated() {
        return !this.equals(this.beanContextChildPeer);
    }
    
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        this.pcSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    public void fireVetoableChange(String propertyName, Object oldValue, Object newValue) throws PropertyVetoException {
        this.vcSupport.fireVetoableChange(propertyName, oldValue, newValue);
    }
    
    public boolean validatePendingSetBeanContext(BeanContext beanContext) {
        return true;
    }
    
    protected void releaseBeanContextResources() {
    }
    
    protected void initializeBeanContextResources() {
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (!this.equals(this.beanContextChildPeer) && !(this.beanContextChildPeer instanceof Serializable)) {
            throw new IOException("BeanContextChildSupport beanContextChildPeer not Serializable");
        }
        objectOutputStream.defaultWriteObject();
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
    }
}
